@@ -17,7 +17,16 @@ check_data_plot <- function(data, defective_factor, subject, factors) {
   if (!is.null(factors) && !all(factors %in% names(data))) {
     stop("factors must name factors in data")
   }
-
+  n_bins <- 4
+  for(fact in factors){
+    if(is.numeric(data[,fact])){
+      if(length(unique(data[,fact])) > 6){
+        quartile_breaks <- quantile(unique(data[,fact]), probs = seq(0, 1, length.out = n_bins + 1), na.rm = TRUE)
+        # Bin the data into quartiles using these breakpoints
+        data[,fact] <- cut(data[,fact], breaks = quartile_breaks, include.lowest = TRUE, labels = paste0("Q", 1:n_bins))
+      }
+    }
+  }
   # Handle subject argument
   if (!is.null(subject)) {
     if(is.numeric(subject)) {
@@ -25,7 +34,7 @@ check_data_plot <- function(data, defective_factor, subject, factors) {
     } else {
       data <- data[data$subjects %in% subject, ]
     }
-    data$subjects <- droplevels(data$subjects)
+    data$subjects <- factor(data$subjects)
   }
 
   # Remove missing or infinite rt
@@ -341,20 +350,20 @@ plot_stat <- function(input, post_predict = NULL, prior_predict = NULL, stat_fun
       }
     }
     if(length(stat_names) > 1){
-      legend(x = legendpos[1], legend = stat_names, lty = lty, col = "black", bty = "n")
+      if(!is.na(legendpos[1])) legend(x = legendpos[1], legend = stat_names, lty = lty, col = "black", bty = "n")
     }
     if (length(data_sources) > 1) {
-      legend(legendpos[2], legend = names(legend_map), lty = 1, col = legend_map, title = "Source", bty = "n", lwd = lwd_map)
+      if(!is.na(legendpos[2])) legend(legendpos[2], legend = names(legend_map), lty = 1, col = legend_map, title = "Source", bty = "n", lwd = lwd_map)
     }
   }
-  return(invisible(summary_df[,!grepl("group_key", colnames(summary_df))]))
+  return(invisible(lapply(summary_sources, function(x) x[,colnames(x) != "group_key"])))
 }
 
 
 
 
 # A small function to compute the defective densities across factor levels
-compute_def_dens <- function(dat, defective_factor, dargs) {
+compute_def_dens <- function(dat, defective_factor, dargs, from = NULL, to = NULL) {
   p_defective <- prop.table(table(dat[[defective_factor]]))
   # We'll call density() on each subset of rt, then multiply by proportion
   # so that the sum across factor levels is 1
@@ -367,7 +376,7 @@ compute_def_dens <- function(dat, defective_factor, dargs) {
       # avoid error
       out[[lev]] <- rep(0, 512)
     } else {
-      dd <- do.call(density, c(list(x = subdat$rt), fix_dots(dargs, density.default, consider_dots = FALSE)))
+      dd <- do.call(density, c(list(x = subdat$rt, from = from, to = to), fix_dots(dargs, density.default, consider_dots = FALSE)))
       out[[lev]] <- dd$y * p_defective[lev]
     }
   }
@@ -434,16 +443,10 @@ plot_density <- function(input, post_predict = NULL, prior_predict = NULL,
 
     # If 'postn' in colnames => multiple sets => need quantiles
     if ("postn" %in% names(src_data)) {
-      # We'll compute from/to from data range
-      rng <-  max(src_data$rt)
       dargs <- switch(
         src_type,
-        "posterior" = add_defaults(posterior_args, to = rng),
-        "prior"     = {
-          rng2 <- quantile(src_data$rt, c(0.975))
-          add_defaults(prior_args, to = rng2)
-        },
-        dots
+        "posterior" = posterior_args,
+        "prior"     = prior_args
       )
 
       splitted <- split(src_data, src_data$group_key)
@@ -451,7 +454,7 @@ plot_density <- function(input, post_predict = NULL, prior_predict = NULL,
       dens_list[[src_name]] <- lapply(splitted, function(dg) {
         postn_splits <- split(dg, dg$postn)
         lapply(postn_splits, function(dsub) {
-          compute_def_dens(dsub, defective_factor, dargs)
+          compute_def_dens(dsub, defective_factor, dargs, from = check$xlim[1]-0.05, to = check$xlim[2] + 0.05)
         })
       })
 
@@ -485,11 +488,9 @@ plot_density <- function(input, post_predict = NULL, prior_predict = NULL,
       }
 
     } else {
-      # Single dataset
-      rng <- quantile(src_data$rt, .99)
-      dargs <- add_defaults(dots, to = rng)
+      dargs <- dots
       splitted <- split(src_data, src_data$group_key)
-      dens_list[[src_name]] <- lapply(splitted, compute_def_dens, defective_factor, dargs)
+      dens_list[[src_name]] <- lapply(splitted, compute_def_dens, defective_factor, dargs, from = check$xlim[1]-0.05, to = check$xlim[2] + 0.05)
 
       if (src_type %in% use_lim) {
         # find max
@@ -530,23 +531,6 @@ plot_density <- function(input, post_predict = NULL, prior_predict = NULL,
   get_range_args <- function(dargs) {
     c(dargs$from, dargs$to)
   }
-
-  # We'll define a small function to figure out the x-grid used for each data source
-  # We do that at plotting time to keep the same 512 points
-  build_x_grid <- function(src_type, src_name, group_key = NULL) {
-    # for postn or single data we found a from/to in a dictionary
-    if (src_type %in% c("data", 'posterior')) {
-      # if data was used, dens_list[['data']][[group_key]] => each level => length=512
-      # but we didn't store the x's. We'll reconstruct from the dots or from
-      rng <- quantile(data_sources[[src_name]]$rt, c(0, 0.99))
-      return(seq(rng[1], rng[2], length.out = 512))
-    } else if (src_type == "prior") {
-      rng2 <- quantile(data_sources[[src_name]]$rt, c(0.001, 0.975))
-      return(seq(rng2[1], rng2[2], length.out = 512))
-    }
-  }
-
-
   for (group_key in unique_group_keys) {
     tmp_dots <- dots
     tmp_posterior_args <- posterior_args
@@ -558,6 +542,8 @@ plot_density <- function(input, post_predict = NULL, prior_predict = NULL,
     do.call(plot, c(list(NA), plot_args))
     legend_map <- c()
     lwd_map <- numeric()
+    x_grid <- seq(xlim[1], xlim[2], length.out = 512)
+
     for (k in 1:length(data_sources)) {
       src_type <- sources[k]
       src_data <- data_sources[[k]]
@@ -579,7 +565,6 @@ plot_density <- function(input, post_predict = NULL, prior_predict = NULL,
         # dens_list[[src_name]][[group_key]] => list of defective_levels => vector of length=512
         cur_dens <- dens_list[[src_name]][[group_key]]
         if (!is.null(cur_dens)) {
-          x_grid <- build_x_grid(src_type, src_name, group_key)
           for (i in seq_along(defective_levels)) {
             lev <- defective_levels[i]
             yvals <- cur_dens[[lev]]
@@ -594,7 +579,6 @@ plot_density <- function(input, post_predict = NULL, prior_predict = NULL,
         # we have postn => look up dens_quants_list
         cur_quants <- dens_quants_list[[src_name]][[group_key]]
         if (!is.null(cur_quants)) {
-          x_grid <- build_x_grid(src_type, src_name, group_key)
           for (i in seq_along(defective_levels)) {
             lev <- defective_levels[i]
             m <- cur_quants[[lev]]
@@ -626,11 +610,15 @@ plot_density <- function(input, post_predict = NULL, prior_predict = NULL,
       }
     }
     # Add legends
-    legend(legendpos[1], legend = defective_levels, lty = line_types, col = "black",
-           title = defective_factor, bty = "n")
+    if(!is.na(legendpos[1])){
+      legend(legendpos[1], legend = defective_levels, lty = line_types, col = "black",
+             title = defective_factor, bty = "n")
+    }
     if (length(data_sources) > 1) {
-      legend(legendpos[2], legend = names(legend_map), lty = 1, col = legend_map, title = "Source", bty = "n",
-             lwd = lwd_map)
+      if(!is.na(legendpos[2])){
+        legend(legendpos[2], legend = names(legend_map), lty = 1, col = legend_map, title = "Source", bty = "n",
+               lwd = lwd_map)
+      }
     }
   }
 }
@@ -730,7 +718,7 @@ plot_cdf <- function(input,
   prior_args <- add_defaults(prior_args, col = c("red", "#800080", "#CC00FF"))
 
   defective_levels <- levels(factor(data_sources[[1]][[defective_factor]]))
-  unique_group_keys <- unique(data_sources[[1]]$group_key)
+  unique_group_keys <- levels(factor(data_sources[[1]]$group_key))
 
   if (is.null(defective_levels) || length(defective_levels) == 0) {
     defective_levels <- "Level1"  # fallback
@@ -945,13 +933,18 @@ plot_cdf <- function(input,
     }
 
     # Factor-level legend
-    legend(legendpos[1], legend=defective_levels, lty=line_types, col="black",
-           title=defective_factor, bty="n")
+    if(!is.na(legendpos[1])){
+      legend(legendpos[1], legend=defective_levels, lty=line_types, col="black",
+             title=defective_factor, bty="n")
+    }
+
 
     # If multiple data sources, show source legend
     if (length(data_sources) > 1) {
-      legend(legendpos[2], legend=names(legend_map), lty=1, col=legend_map,
-             title="Source", bty="n", lwd = lwd_map)
+      if(!is.na(legendpos[2])){
+        legend(legendpos[2], legend=names(legend_map), lty=1, col=legend_map,
+               title="Source", bty="n", lwd = lwd_map)
+      }
     }
   } # end for each group_key
 
