@@ -52,8 +52,6 @@ compare <- function(sList,stage="sample",filter=NULL,use_best_fit=TRUE,
     IC <- -(IC - min(IC))/2
     exp(IC)/sum(exp(IC))
   }
-  if(!is.null(list(...)$subject)) subject <- list(...)$subject
-
   if (is.numeric(filter)) defaultsf <- filter[1] else defaultsf <- 0
   sflist <- as.list(setNames(rep(defaultsf,length(sList)),names(sList)))
   if (is.list(filter)) for (i in names(filter))
@@ -61,7 +59,7 @@ compare <- function(sList,stage="sample",filter=NULL,use_best_fit=TRUE,
   dots <- add_defaults(list(...), group_only = FALSE)
   ICs <- setNames(vector(mode="list",length=length(sList)),names(sList))
   for (i in 1:length(ICs)) ICs[[i]] <- IC(sList[[i]],stage=stage,
-                                          filter=sflist[[i]],use_best_fit=use_best_fit,subject=NULL,print_summary=FALSE,
+                                          filter=sflist[[i]],use_best_fit=use_best_fit,subject=list(...)$subject,print_summary=FALSE,
                                           group_only = dots$group_only)
   ICs <- data.frame(do.call(rbind,ICs))
   DICp <- getp(ICs$DIC)
@@ -355,6 +353,8 @@ compare_subject <- function(sList,stage="sample",filter=0,use_best_fit=TRUE,
                             print_summary=TRUE,digits=3) {
   if(is(sList, "emc")) sList <- list(sList)
   subjects <- names(sList[[1]][[1]]$data)
+  is_single <- sapply(sList, function(x) return(x[[1]]$type == "single"))
+  if(any(!is_single)) warning("subject-by-subject comparison is best done with models of type `single`")
   out <- setNames(vector(mode="list",length=length(subjects)),subjects)
   for (i in subjects) out[[i]] <- compare(sList,subject=i,BayesFactor=FALSE,
                                           stage=stage,filter=filter,use_best_fit=use_best_fit,print_summary=FALSE)
@@ -524,5 +524,79 @@ get_posterior_quantiles <- function(x, probs = c(0.025, .5, .975)){
   return(summ$quantiles)
 }
 
+#' Model Averaging
+#'
+#' Computes model weights and a Bayes factor by comparing two groups of models based on their
+#' Information Criterion (IC) values. The function works with either numeric vectors or data
+#' frames containing multiple IC measures (e.g., MD, BPIC, DIC).
+#'
+#' When provided with numeric vectors, it computes the weights for the two groups by first
+#' converting the IC values into relative weights and then normalizing them. When provided with
+#' a data frame, it assumes that the data frame is the output of a call to `compare`
+#' and applies averaging to each IC metric
+#'
+#' @param IC_for A numeric vector or the output of `compare`
+#' @param IC_against A numeric vector or the output of `compare`
+#'
+#' @return A \code{data.frame} with the following columns:
+#'   \describe{
+#'     \item{\code{wFor}}{The aggregated weight of the models in favor.}
+#'     \item{\code{wAgainst}}{The aggregated weight of the models against.}
+#'     \item{\code{Factor}}{The Bayes factor (ratio of \code{wFor} to \code{wAgainst}).}
+#'   }
+#'   If \code{IC_for} is a data frame, a matrix with rows corresponding to each IC measure is returned.
+#'
+#' @examples
+#' # First set up some example models (normally these would be alternative models)
+#' samples_LNR2 <- subset(samples_LNR, length.out = 45)
+#' samples_LNR3 <- subset(samples_LNR, length.out = 40)
+#' samples_LNR4 <- subset(samples_LNR, length.out = 35)
+#'
+#' # Run compare on them, BayesFactor = F is set for speed.
+#' ICs <- compare(list(S1 = samples_LNR, S2 = samples_LNR2,
+#'                     S3 = samples_LNR3, S4 = samples_LNR4), BayesFactor = FALSE)
+#'
+#' # Model averaging can either be done with a vector of ICs:
+#' model_averaging(ICs$BPIC[1:2], ICs$BPIC[2:4])
+#'
+#' # Or the output of compare:
+#' model_averaging(ICs[1:2,], ICs[3:4,])
+#'
+#' @export
+model_averaging <- function(IC_for, IC_against) {
+  if(is.null(IC_for)) return(NULL)
 
+  if(is.data.frame(IC_for)){
+    # Recursive call to make it work with the output of compare
+    MD <- model_averaging(IC_for$MD, IC_against$MD)
+    BPIC <- model_averaging(IC_for$BPIC, IC_against$BPIC)
+    DIC <- model_averaging(IC_for$DIC, IC_against$DIC)
+    return(rbind(MD = MD, BPIC = BPIC, DIC = DIC))
+  }
+  # Combine the IC values from both groups
+  all_IC <- c(IC_for, IC_against)
+
+  # Find the smallest IC value (for numerical stability)
+  min_IC <- min(all_IC)
+
+  # Compute the unnormalized weights
+  unnorm_weights <- exp(-0.5 * (all_IC - min_IC))
+
+  # Normalize weights so they sum to 1
+  weights <- unnorm_weights / sum(unnorm_weights)
+
+  # Separate the weights for the two groups
+  weight_for <- sum(weights[seq_along(IC_for)])
+  weight_against <- sum(weights[(length(IC_for) + 1):length(all_IC)])
+
+  # Compute the Bayes factor: evidence in favor relative to against
+  bayes_factor <- weight_for / weight_against
+
+  # Return the results as a data frame
+  return(data.frame(
+    wFor = weight_for,
+    wAgainst = weight_against,
+    Factor = bayes_factor
+  ))
+}
 
